/*
 *	binkleyforce -- unix FTN mailer project
 *	
 *	Copyright (c) 1998-2000 Alexander Belkin, 2:5020/1398.11
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	$Id: outb_bsy.c,v 1.2 2000/05/12 21:32:34 adb Exp $
 */

#include "includes.h"
#include "confread.h"
#include "logger.h"
#include "util.h"
#include "outbound.h"

#define BSY_OK		0x00
#define BSY_EXIST	0x01
#define BSY_ERROR	0x02
#define BSY_OURBSY	0x03

typedef struct bsylist {
	s_faddr addr;
	char *bsyname_4d;
	char *bsyname_domain;
	char *bsyname_amiga;
	struct bsylist *next;
} s_bsylist;

s_bsylist *bsylist = NULL;

/* ========================================================================= */
/* PRIVATE part                                                              */
/* ========================================================================= */

static pid_t out_bsy_read(const char *bsyname)
{
	FILE *fp = NULL;
	pid_t pid = 0;
	
	ASSERT(bsyname != NULL);

	if( (fp = fopen(bsyname, "rt")) )
	{
		if( fscanf(fp, "%d", &pid) != 1 ) pid = 0;
		fclose(fp);
	}
	
	DEB((D_OUTBOUND, "out_bsy_read: BSY file \"%s\", PID = %d", bsyname, pid));
	
	return pid;
}

/* ------------------------------------------------------------------------- */
/* Return codes:                                                             */
/*   BSY_OK                                                                  */
/*   BSY_EXIST                                                               */
/*   BSY_ERROR                                                               */
/*   BSY_OURBSY                                                              */
/* ------------------------------------------------------------------------- */
static int out_bsy_checkfile(const char *bsyname)
{
	pid_t pid = 0;
	struct stat st;
	
	ASSERT(bsyname != NULL);
	
	if( stat(bsyname, &st) && errno == ENOENT )
		{ return BSY_OK; }

	if( (pid = out_bsy_read(bsyname)) == 0 )
		{ return (errno == ENOENT) ? BSY_OK : BSY_ERROR; }

	if( pid == getpid() )
		{ return BSY_OURBSY; /* Should we unlink it? */ }
	
	if( kill(pid, 0) == -1 && errno == ESRCH )
	{
		/*
		 *  that process no longer exists,
		 *  try to remove lock file
		 */
		log("found stale BSY file with PID %d", pid);
		if( unlink(bsyname) == 0 )
		{
			return BSY_OK;
		}
		else
		{
			logerr("can't remove bsy file \"%s\"", bsyname);
			return BSY_ERROR;
		}
	}

	return BSY_EXIST; /* locking process still lives */
}

static int out_bsy_createlink(const char *bsyname, const char *tmpname)
{
	int rc, fd;
	char buf[32];

	ASSERT(bsyname != NULL && tmpname != NULL);

	if( (fd = open(tmpname, O_CREAT | O_RDWR, 0644)) < 0 )
	{
		/* Don't log any errors, really we can have
		 * no outbound directory for this address */
		return BSY_ERROR;
	}

	chmod(tmpname, 0644);

	sprintf(buf, "%10d\n", (int)getpid());
	
	if( write(fd, (char *)buf, strlen(buf)) != strlen(buf) )
	{
		logerr("can't write PID to temp. BSY file \"%s\"", tmpname);
		unlink(tmpname);
		return BSY_ERROR;
	}
	
	if( link(tmpname, bsyname) == 0 )
	{
		unlink(tmpname);
		return BSY_OK;
	}

	/* Can't create link */
	if( errno == EEXIST )
	{
		if( (rc = out_bsy_checkfile(bsyname)) != BSY_OK )
		{
			if( rc == BSY_OURBSY ) rc = BSY_OK;
		}
		else
		{
			rc = link(tmpname, bsyname) ? BSY_ERROR : BSY_OK;
		}
	}
	else rc = BSY_ERROR;

	unlink(tmpname);
	return rc;
}

static int out_bsy_createfile(const char *bsyname)
{
	int rc;
	char *tmpname, *p;

	ASSERT(bsyname != NULL);
		
	tmpname = xstrcpy(bsyname);
	if( (p = strrchr(tmpname, DIRSEPCHR)) && p != tmpname )	{ *++p = '\0'; }
	else { free(tmpname); return BSY_ERROR; }
	tmpname = xstrcat(tmpname, "bfXXXXXX");

	if( (p = mktemp(tmpname)) == NULL )
	{
		logerr("can't generate unique file name from \"%s\"", tmpname);
		free(tmpname); return BSY_ERROR;
	}

	rc = out_bsy_createlink(bsyname, p);

	DEB((D_OUTBOUND, "out_bsy_createfile: createlink(\"%s\", \"%s\"), rc = %d",
		bsyname, p, rc));
	
	free(tmpname);
	
	return rc;
}

static int out_bsy_unlinkfile(const char *bsyname)
{
	pid_t pid = 0;
	
	DEB((D_OUTBOUND, "out_bsy_unlinkfile: want unlink \"%s\"", bsyname));
	
	if( (pid = out_bsy_read(bsyname)) == 0 )
		{ return 1; }
	
	if( pid == getpid() && unlink(bsyname) == -1 )
		{ return 1; }
	
	return 0;
}

/* ========================================================================= */
/* PUBLIC part                                                               */
/* ========================================================================= */

int out_bsy_check(s_faddr addr)
{
	bool isfailed = FALSE;
	char *bsyname_4d = NULL;
	char *bsyname_domain = NULL;
	char *bsyname_amiga = NULL;

	bsyname_4d     = out_getname_4d(addr);
	bsyname_domain = out_getname_domain(addr);
	bsyname_amiga  = out_getname_amiga(addr);

	if( bsyname_4d )
		{ bsyname_4d = xstrcat(bsyname_4d, ".bsy"); }
	if( bsyname_domain )
		{ bsyname_domain = xstrcat(bsyname_domain, ".bsy"); }
	if( bsyname_amiga )
		{ bsyname_amiga = xstrcat(bsyname_amiga, ".bsy"); }

	DEB((D_OUTBOUND, "out_bsy_check: bsyname_4d = \"%s\"", bsyname_4d));
	DEB((D_OUTBOUND, "out_bsy_check: bsyname_domain = \"%s\"", bsyname_domain));
	DEB((D_OUTBOUND, "out_bsy_check: bsyname_amiga = \"%s\"", bsyname_amiga));
	
	/* Check for BSY files in all outbounds */
	if( !isfailed && bsyname_4d && out_bsy_checkfile(bsyname_4d) == BSY_EXIST )
		{ isfailed = TRUE; }
	if( !isfailed && bsyname_domain && out_bsy_checkfile(bsyname_domain) == BSY_EXIST )
		{ isfailed = TRUE; }
	if( !isfailed && bsyname_amiga && out_bsy_checkfile(bsyname_amiga) == BSY_EXIST )
		{ isfailed = TRUE; }
	
	if( bsyname_4d )
		{ free(bsyname_4d); bsyname_4d = NULL; }
	if( bsyname_domain )
		{ free(bsyname_domain); bsyname_domain = NULL; }
	if( bsyname_amiga )
		{ free(bsyname_amiga); bsyname_amiga = NULL; }
	
	return isfailed;
}

int out_bsy_lock(s_faddr addr)
{
	char *bsyname_4d = NULL;
	char *bsyname_domain = NULL;
	char *bsyname_amiga = NULL;
	bool locked_4d = FALSE;
	bool locked_domain = FALSE;
	bool locked_amiga = FALSE;
	s_bsylist bsytmp, **bsylst;
	bool isfailed = FALSE;
#ifdef DEBUG
	char abuf[BF_MAXADDRSTR+1];
#endif

	DEB((D_OUTBOUND, "out_bsy_lock: lock address %s", ftn_addrstr(abuf, addr)));
	
	bsyname_4d     = out_getname_4d(addr);
	bsyname_domain = out_getname_domain(addr);
	bsyname_amiga  = out_getname_amiga(addr);

	if( bsyname_4d )
		{ bsyname_4d = xstrcat(bsyname_4d, ".bsy"); }
	if( bsyname_domain )
		{ bsyname_domain = xstrcat(bsyname_domain, ".bsy"); }
	if( bsyname_amiga )
		{ bsyname_amiga = xstrcat(bsyname_amiga, ".bsy"); }

	DEB((D_OUTBOUND, "out_bsy_lock: bsyname_4d = \"%s\"", bsyname_4d));
	DEB((D_OUTBOUND, "out_bsy_lock: bsyname_domain = \"%s\"", bsyname_domain));
	DEB((D_OUTBOUND, "out_bsy_lock: bsyname_amiga = \"%s\"", bsyname_amiga));
	
	/* Check for BSY files in all outbounds */
	if( !isfailed && bsyname_4d && out_bsy_checkfile(bsyname_4d) == BSY_EXIST )
		{ isfailed = TRUE; }
	if( !isfailed && bsyname_domain && out_bsy_checkfile(bsyname_domain) == BSY_EXIST )
		{ isfailed = TRUE; }
	if( !isfailed && bsyname_amiga && out_bsy_checkfile(bsyname_amiga) == BSY_EXIST )
		{ isfailed = TRUE; }

	DEB((D_OUTBOUND, "out_bsy_lock: lock check result: %s", isfailed?"Locked":"Not locked"));
		
	/* Create BSY file in 4D outbound */
	if( !isfailed && bsyname_4d )
	{
		switch( out_bsy_createfile(bsyname_4d) ) {
		case BSY_OK:	locked_4d = TRUE; break;
		case BSY_EXIST:	isfailed = TRUE; break;
		case BSY_ERROR:	locked_4d = FALSE; break;
		}
	}
	
	/* Create BSY file in domain outbound */
	if( !isfailed && bsyname_domain )
	{
		switch( out_bsy_createfile(bsyname_domain) ) {
		case BSY_OK:	locked_domain = TRUE; break;
		case BSY_EXIST:	isfailed = TRUE; break;
		case BSY_ERROR:	locked_domain = FALSE; break;
		}
	}

	/* Create BSY file in "AmigaDos" outbound */
	if( !isfailed && bsyname_amiga )
	{
		switch( out_bsy_createfile(bsyname_amiga) ) {
		case BSY_OK:	locked_amiga = TRUE; break;
		case BSY_EXIST:	isfailed = TRUE; break;
		case BSY_ERROR:	locked_amiga = FALSE; break;
		}
	}
	
	DEB((D_OUTBOUND, "out_bsy_lock: lock create result: %s", isfailed?"Failed":"Successful"));

	/* Somebody else create BSY before us?! */
	if( isfailed )
	{
		if( locked_4d )
			{ out_bsy_unlinkfile(bsyname_4d); }
		if( locked_domain )
			{ out_bsy_unlinkfile(bsyname_domain); }
		if( locked_amiga )
			{ out_bsy_unlinkfile(bsyname_amiga); }
	}
	else /* Successful locking */
	{
		memset(&bsytmp, '\0', sizeof(s_bsylist));
		
		bsytmp.addr = addr;
		
		if( locked_4d )
			{ bsytmp.bsyname_4d = bsyname_4d; bsyname_4d = NULL; }
		if( locked_domain )
			{ bsytmp.bsyname_domain = bsyname_domain; bsyname_domain = NULL; }
		if( locked_amiga )
			{ bsytmp.bsyname_amiga = bsyname_amiga; bsyname_amiga = NULL; }
		
		/* Add new entry to the bsylist */
		for( bsylst = &bsylist; *bsylst; bsylst = &(*bsylst)->next )
			{ /* EMPTY LOOP */ }
		
		(*bsylst) = (s_bsylist*)xmalloc(sizeof(s_bsylist));
		memcpy(*bsylst, &bsytmp, sizeof(s_bsylist));
	}
	
	if( bsyname_4d )
		{ free(bsyname_4d); bsyname_4d = NULL; }
	if( bsyname_domain )
		{ free(bsyname_domain); bsyname_domain = NULL; }
	if( bsyname_amiga )
		{ free(bsyname_amiga); bsyname_amiga = NULL; }
	
	return isfailed;
}

int out_bsy_unlockall(void)
{
	s_bsylist *ptrl, *next;
#ifdef DEBUG
	char abuf[BF_MAXADDRSTR+1];
#endif
	
	for( ptrl = bsylist; ptrl; ptrl = next )
	{
		next = ptrl->next;
		
		DEB((D_OUTBOUND, "out_bsy_unlockall: unlock address %s",
			ftn_addrstr(abuf, ptrl->addr)));
		
		if( ptrl->bsyname_4d )
		{
			out_bsy_unlinkfile(ptrl->bsyname_4d);
			free(ptrl->bsyname_4d);
		}
		if( ptrl->bsyname_domain )
		{
			out_bsy_unlinkfile(ptrl->bsyname_domain);
			free(ptrl->bsyname_domain);
		}
		if( ptrl->bsyname_amiga )
		{
			out_bsy_unlinkfile(ptrl->bsyname_amiga);
			free(ptrl->bsyname_amiga);
		}
		free(ptrl);
	}
	
	bsylist = NULL;
	
	return 0;
}
