/*
 *	binkleyforce -- unix FTN mailer project
 *	
 *	Copyright (c) 1998-2000 Alexander Belkin, 2:5020/1398.11
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	$Id: outb_scan.c,v 1.6 2000/08/28 11:36:15 adb Exp $
 */

#include "includes.h"
#include "confread.h"
#include "logger.h"
#include "util.h"
#include "outbound.h"

/* ---------------------------------------------------------- */
/* different outbound files - their extensions and constants  */
/* ---------------------------------------------------------- */
s_exttab outtab[] =
{
	{ ".iut", TYPE_NETMAIL, FLAVOR_IMMED,  ACTION_UNLINK   },
	{ ".cut", TYPE_NETMAIL, FLAVOR_CRASH,  ACTION_UNLINK   },
	{ ".dut", TYPE_NETMAIL, FLAVOR_DIRECT, ACTION_UNLINK   },
	{ ".out", TYPE_NETMAIL, FLAVOR_NORMAL, ACTION_UNLINK   },
	{ ".hut", TYPE_NETMAIL, FLAVOR_HOLD,   ACTION_UNLINK   },
	{ ".ikt", TYPE_NETMAIL, FLAVOR_IMMED,  ACTION_UNLINK   },
	{ ".ckt", TYPE_NETMAIL, FLAVOR_CRASH,  ACTION_UNLINK   },
	{ ".dkt", TYPE_NETMAIL, FLAVOR_DIRECT, ACTION_UNLINK   },
	{ ".pkt", TYPE_NETMAIL, FLAVOR_NORMAL, ACTION_UNLINK   },
	{ ".hkt", TYPE_NETMAIL, FLAVOR_HOLD,   ACTION_UNLINK   },
	{ ".ilo", TYPE_FLOFILE, FLAVOR_IMMED,  ACTION_NOTHING  },
	{ ".clo", TYPE_FLOFILE, FLAVOR_CRASH,  ACTION_NOTHING  },
	{ ".dlo", TYPE_FLOFILE, FLAVOR_DIRECT, ACTION_NOTHING  },
	{ ".flo", TYPE_FLOFILE, FLAVOR_NORMAL, ACTION_NOTHING  },
	{ ".hlo", TYPE_FLOFILE, FLAVOR_HOLD,   ACTION_NOTHING  },
	{ ".req", TYPE_REQUEST, FLAVOR_NONE,   ACTION_NOTHING  },
	{  NULL,  0,            0,             0               }
};
	
/* ----------------------------------------------------------- */
/* Extensions of some files.. It's best to know about them :)  */
/* ----------------------------------------------------------- */
s_exttab exttab[] =
{
	{ ".tic", TYPE_TICFILE, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".su?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".mo?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".tu?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".we?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".th?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".fr?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{ ".sa?", TYPE_ARCMAIL, FLAVOR_NORMAL, ACTION_TRUNCATE },
	{  NULL,  0,            0,             0               }
};

/* ------------------------------------------------------------------------- */
/* Devide full outbound path on "root" directory, there other oubound        */
/* directories created, and main outbound directory name. If can not get     */
/* some of above information - return non-zero vaule.                        */
/*   WARNING: *out_root and *out_main must be freed!                         */
/* ------------------------------------------------------------------------- */
int out_getroot(const char *outbound, char **out_root, char **out_main)
{
	char *p, *outb;

	ASSERT(outbound != NULL && out_root != NULL && out_main != NULL);

	*out_root = NULL;
	*out_main = NULL;
	
	outb = (char *)xstrcpy(outbound);

	p = outb + strlen(outb) - 1;
	if( *p == DIRSEPCHR ) *p = '\0';
	
	/* Ohh.. where is no full path for outbound directory */
	if( (p=strrchr(outb, DIRSEPCHR)) == NULL )
		{ free(outb); return(1); }
	
	*p = '\0';
	*out_main = xstrcpy(p+1);
	*out_root = xstrcat(outb, "/");
	
	DEB((D_OUTBOUND, "out_getroot: out_root \"%s\", out_main \"%s\"",
		*out_root, *out_main));
		
	return(0);
}

/* ------------------------------------------------------------------------- */
/* Scan Binkley Style Outbound directory $dir_name, for each file for        */
/* address listed in $fa_list call proc() handler.                           */
/* ------------------------------------------------------------------------- */
static int out_scandir_bso(void (*proc)(char*, s_faddr, const char*, bool),
                    char *dst, s_falist *fa_list, s_faddr *addr, char *path,
                    int point)
{
	DIR *dir;
	char *p = NULL;
	struct dirent *ent = NULL;
	s_falist *alst = NULL;

	ASSERT(dst != NULL && addr != NULL && path != NULL);
	
	DEB((D_OUTBOUND, "out_scandir_bso: BSO: scan dir \"%s\"", path));
	
	if( (dir=opendir(path)) == NULL )
	{
		logerr("can't open outbound directory \"%s\"", path);
		return(1);
	}
	
	while( (ent=readdir(dir)) != NULL )
	{
		if( strlen(ent->d_name) == 12 &&
		    strspn(ent->d_name, "0123456789abcdefABCDEF") == 8 )
		{
			if( point == 0 && strncasecmp(ent->d_name+8, ".PNT", 4) == 0 )
			{
				DEB((D_OUTBOUND, "scan_dir: BSO: found point directory \"%s\"",
					ent->d_name));
					
				/*
				 * Congratulation, we found point's direcory
				 */
				sscanf(ent->d_name, "%04x%04x", &addr->net, &addr->node);
				for( alst = fa_list; alst; alst = alst->next )
					if( addr->net  == alst->addr.net
					 && addr->node == alst->addr.node )
						break;
				
				if( alst || fa_list == NULL )
				{
					p = string_concat(path, ent->d_name, "/", NULL);
					out_scandir_bso(proc, dst, fa_list, addr, p, 1);
					if( p ) { free(p); p = NULL; }
				}
			}
			else
			{
				if( point )
				{
					sscanf(ent->d_name, "%08x", &addr->point);
				}
				else
				{
					addr->point = 0;
					sscanf(ent->d_name, "%04x%04x", &addr->net, &addr->node);
				}

				/* Check, does we really need in this file? */
				for( alst = fa_list; alst; alst = alst->next )
					if( ftn_addrcomp(*addr, alst->addr) == 0 )
						break;
				
				if( alst || fa_list == NULL )
				{
					char *tmp = string_concat(path, ent->d_name, NULL);
					(*proc)(dst, *addr, tmp, 0);
					free(tmp);
				}
			}
		}
	}
	
	closedir(dir);
	
	return(0);
}

/* ------------------------------------------------------------------------- */
/* Parse file name from AmigaDos Style Outbound, return zero on success      */
/* ------------------------------------------------------------------------- */
int out_parsename_aso(const char *name, s_faddr *addr)
{
	memset(addr, '\0', sizeof(s_faddr));
	if( sscanf(name, "%d.%d.%d.%d.%*s", &addr->zone,
	    &addr->net, &addr->node, &addr->point) == 4 )
		{ return 0; }
	else
		{ memset(addr, '\0', sizeof(s_faddr)); return 1; }
}

/* ------------------------------------------------------------------------- */
/* Scan AmigaDos Style 4D Outbound directory $dir_name, for each file for    */
/* address listed in $fa_list call proc() handler.                           */
/* ------------------------------------------------------------------------- */
static int out_scandir_aso(void (*proc)(char*, s_faddr, const char*, bool),
                           char *dst, s_falist *fa_list, char *path)
{
	DIR *dir;
	struct dirent *ent = NULL;
	s_falist *alst = NULL;
	s_faddr addr = { FALSE, 0, 0, 0, 0, "" };
#ifdef DEBUG
	char abuf[BF_MAXADDRSTR+1];
#endif
		
	ASSERT(dst != NULL && path != NULL);

	DEB((D_OUTBOUND, "out_scandir_aso: scan dir \"%s\"", path));
	
	if( (dir = opendir(path)) == NULL )
	{
		logerr("can't open outbound directory \"%s\"", path);
		return(1);
	}
	
	while( (ent = readdir(dir)) )
	{
		memset(&addr, '\0', sizeof(s_faddr));
		
		if( out_parsename_aso(ent->d_name, &addr) == 0 )
		{
			DEB((D_OUTBOUND, "out_scandir_aso: file \"%s\" for address %s",
				ent->d_name, ftn_addrstr(abuf, addr)));

			for( alst = fa_list; alst; alst = alst->next )
				if( ftn_addrcomp(addr, alst->addr) == 0 )
					break;
			
			if( alst || fa_list == NULL )
			{
				char *tmp = string_concat(path, ent->d_name, NULL);
				(*proc)(dst, addr, tmp, 1);
				free(tmp);
			}
		}
	}
	
	closedir(dir);
	
	return(0);
}

/* ------------------------------------------------------------------------- */
/* Scan outbound for addresses fa_list, use handler proc                     */
/* ------------------------------------------------------------------------- */
int out_scan(void (*proc)(char*, s_faddr, const char*, bool),
             char *dst, s_falist *fa_list)
{
	DIR *dir;
	struct dirent *ent;
	s_falist *alst = NULL;
	s_faddr addr = { FALSE, 0, 0, 0, 0, "" };
	s_cval_entry *cfptr;
	char *path = NULL;
	char *outbound = NULL;
	char *out_root = NULL;	/* Root for outbound directories tree */
	char *out_main = NULL;	/* Main outbound directory name       */
	int len;
	
	ASSERT(proc != NULL && dst != NULL);

	/*
	 * 1st - try out 4D outbound
	 */
	outbound = conf_string(cf_outbound_directory);
	
	if( outbound && *outbound
	 && out_getroot(outbound, &out_root, &out_main) == 0
	 && out_root && *out_root && out_main && *out_main )
	{
		DEB((D_OUTBOUND, "out_scan: BSO: open dir \"%s\"", out_root));

		if( (dir=opendir(out_root)) == NULL )
		{
			logerr("can't open outbound directory \"%s\"", out_root);
		}
		else
		{
			/* Scan outbound (only 4D) */
			while( (ent=readdir(dir)) != NULL )
			{
				memset(&addr, '\0', sizeof(s_faddr));
				if( strcmp(ent->d_name, out_main) == 0 )
				{
					/* Our main outbound */
					cfptr = conf_first(cf_address);
					if( cfptr )
						addr.zone = cfptr->d.falist.addr.zone;
					else
						addr.zone = 0;
				}
				else if( strncmp(ent->d_name, out_main, len=strlen(out_main)) == 0
				      && ent->d_name[len] == '.'
				      && strspn(ent->d_name+len+1,"0123456789abcdefABCDEF") == 3 )
				{
					/* Ooutbound with zone ext. (like /outbound.308/) */
					sscanf(ent->d_name+len+1, "%03x", &addr.zone);
				}

				if( addr.zone )
				{
					for( alst = fa_list; alst; alst = alst->next )
						if( addr.zone == alst->addr.zone || alst->addr.zone == -1 )
							break;
					
					if( alst || fa_list == NULL )
					{
						path = string_concat(out_root, ent->d_name, "/", NULL);
						out_scandir_bso(proc, dst, fa_list, &addr, path, 0);
						if( path ) { free(path); path = NULL; }
					}
				}
			}
			closedir(dir);
			DEB((D_OUTBOUND, "out_scan: BSO: dir closed"));
		}
	}
	
	if( out_root ) { free(out_root); out_root = NULL; }
	if( out_main ) { free(out_main); out_main = NULL; }
	
	/*
	 *  2th - Scan domain outbounds
	 */
	for( cfptr = conf_first(cf_domain); cfptr; cfptr = conf_next(cfptr) )
	{
		memset(&addr, '\0', sizeof(s_faddr));
		for( alst = fa_list; alst; alst = alst->next )
		{
			if( cfptr->d.domain.zone == alst->addr.zone
			 || alst->addr.zone == -1 )
			{
				addr.zone = cfptr->d.domain.zone;
				out_scandir_bso(proc, dst, fa_list, &addr,
						cfptr->d.domain.path, 0);
				break;
			}
		}
	}

	/*
	 *  3th - Scan AmigaDOS outbound
	 */
	outbound = conf_string(cf_amiga_outbound_directory);
	
	if( outbound && *outbound )
		out_scandir_aso(proc, dst, fa_list, outbound);
	
	return(0);
}
