/*
 *	binkleyforce -- unix FTN mailer project
 *	
 *	Copyright (c) 1998-2000 Alexander Belkin, 2:5020/1398.11
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	$Id: bfstat.c,v 1.15 2000/10/28 11:23:03 adb Exp $
 */

#include "includes.h"
#include "confread.h"
#include "version.h"
#include "logger.h"
#include "util.h"
#include "outbound.h"

/*
 *  Our fake expressions checker. Allways return FALSE (?)
 */
int eventexpr(s_expr *expr)
{
	return 0;
}

static void usage(void)
{
	printf_usage("outbound viewer",
		"usage: bfstat [-afhprst] [-n<number>]\n"
		"\n"
		"options:\n"
		"  -a                sort by FTN address (default)\n"
		"  -f                disable queue sorting\n"
		"  -h                print this help message\n"
		"  -n <number>       don't print more than <number> systems\n"
		"  -p                print sizes in human readable format\n"
		"  -r                reverse order while sorting\n"
		"  -s                sort by total files size\n"
		"  -t                disable total sizes printing\n"
		"\n"
	);
}

int main(int argc, char *argv[])
{
	s_sysqueue queue;
	char abuf[BF_MAXADDRSTR+1];
	int i;
	bool sort_bysize = FALSE;
	bool sort_byaddr = TRUE;
	bool sort_reverse = FALSE;
	bool disable_total_sizes = FALSE;
	bool humansizes = FALSE;
	int nodeslimit = 0;
	char c;
	
	while( (c = getopt(argc, argv, "afhn:prst")) != EOF )
	{
		switch( c ) {
		case 'h':
			usage();
			exit(0);
		case 'f':
			sort_bysize = FALSE;
			sort_byaddr = FALSE;
			sort_reverse = FALSE;
			break;
		case 'p':
			humansizes = TRUE;
			break;
		case 't':
			disable_total_sizes = TRUE;
			break;
		case 'r':
			sort_reverse = TRUE;
			break;
		case 's':
			sort_byaddr = FALSE;
			sort_bysize = TRUE;
			break;
		case 'a':
			sort_byaddr = TRUE;
			sort_bysize = FALSE;
			break;
		case 'n':
			if( ISDEC(optarg) )
				nodeslimit = atoi(optarg);
			break;
		default:
			usage();
			exit(1);
		}
	}

	memset(&queue, '\0', sizeof(s_sysqueue));
	
	/* Initialise random number generation */
	(void)srand((unsigned)time(0));
	/* Initialise current locale */
	(void)setlocale(LC_ALL, "");
	
	if( conf_readconf(conf_getconfname(), 0) )
		exit(1);

	out_scan(out_handle_sysqueue, (char*)&queue, NULL);
	
	if( queue.sysnum > 1 && (sort_byaddr || sort_bysize) )
	{
		int opts = sort_byaddr ? QUEUE_SORT_ADDRESS
		         : sort_bysize ? QUEUE_SORT_SIZE : 0;
		
		out_sysqueue_sort(&queue, opts | (sort_reverse ? QUEUE_SORT_REVERSE : 0));
	}

	if( queue.sysnum > 0 )
	{
		char sbuf[4][10];
		size_t size_netmail = 0;
		size_t size_arcmail = 0;
		size_t size_other   = 0;
		int printnum = nodeslimit ? MIN(nodeslimit, queue.sysnum) : queue.sysnum;
		
		printf("Address                   Netmail    Arcmail    Other      Flavors\n");
		printf("-------------------------------------------------------------------\n");

		for( i = 0; i < printnum; i++ )
		{
			if( !disable_total_sizes )
			{
				size_netmail += queue.systab[i].netmail_size;
				size_arcmail += queue.systab[i].arcmail_size;
				size_other   += queue.systab[i].request_size
				              + queue.systab[i].files_size;
			}
			
			if( humansizes )
			{
			printf("%-25s %-10s %-10s %-10s %c%c%c%c%c\n",
				ftn_addrstr(abuf, queue.systab[i].node.addr),
				string_humansize(sbuf[0], queue.systab[i].netmail_size),
				string_humansize(sbuf[1], queue.systab[i].arcmail_size),
				string_humansize(sbuf[2], queue.systab[i].request_size + queue.systab[i].files_size),
				(queue.systab[i].flavors & FLAVOR_HOLD  ) ? 'H' : '.',
				(queue.systab[i].flavors & FLAVOR_NORMAL) ? 'N' : '.',
				(queue.systab[i].flavors & FLAVOR_DIRECT) ? 'D' : '.',
				(queue.systab[i].flavors & FLAVOR_CRASH ) ? 'C' : '.',
				(queue.systab[i].flavors & FLAVOR_IMMED ) ? 'I' : '.');
			}
			else
			{
			printf("%-25s %-10ld %-10ld %-10ld %c%c%c%c%c\n",
				ftn_addrstr(abuf, queue.systab[i].node.addr),
				(long)(queue.systab[i].netmail_size),
				(long)(queue.systab[i].arcmail_size),
				(long)(queue.systab[i].request_size + queue.systab[i].files_size),
				(queue.systab[i].flavors & FLAVOR_HOLD  ) ? 'H' : '.',
				(queue.systab[i].flavors & FLAVOR_NORMAL) ? 'N' : '.',
				(queue.systab[i].flavors & FLAVOR_DIRECT) ? 'D' : '.',
				(queue.systab[i].flavors & FLAVOR_CRASH ) ? 'C' : '.',
				(queue.systab[i].flavors & FLAVOR_IMMED ) ? 'I' : '.');
			}
		}

		if( !disable_total_sizes )
		{
			printf("-------------------------------------------------------------------\n");
			
			if( humansizes )
			{
				printf("TOTAL %3d systems         %-10s %-10s %-10s %-10s\n",
					printnum,
					string_humansize(sbuf[0], size_netmail),
					string_humansize(sbuf[1], size_arcmail),
					string_humansize(sbuf[2], size_other),
					string_humansize(sbuf[3], size_netmail+size_arcmail+size_other));
			}
			else
			{
				printf("TOTAL %3d systems         %-10ld %-10ld %-10ld %-10ld\n",
					printnum,
					(long)size_netmail, (long)size_arcmail, (long)size_other,
					(long)(size_netmail + size_arcmail + size_other));
			}
		}
	}
	else
		printf("Outbound queue is empty\n");
	
	fflush(stdout);
	
	deinit_sysqueue(&queue);
	deinit_conf();

	exit(0);
}
